library(Matrix)
library(tseries)
library(ada)
source("http://lu-library.googlecode.com/svn/trunk/evaluation.r")

#adaboost
train_adaboost <- function(splitid, nfeat, iter = 20, loss = "exponential") {
	testidx = which(splits[,splitid]==1)
	trainidx = which(splits[,splitid]==0)
	print(splitid)
	
	trmat <- as.matrix(nfeat[trainidx,])
	tsmat <- as.data.frame(nfeat[testidx,])
	
	set.seed(0)
	predictions <-  matrix(0, nrow = length(testidx), ncol = length(event.ids))
	for(eventid in 1:length(event.ids)) {
		trlabels = labels[trainidx, eventid]
		set.seed(0) 
		model <- ada(x = trmat, y = factor(trlabels), loss=loss, iter = iter)
		pred<- predict(model, tsmat, type = "prob")
		tsprediction <- pred[,2]
		predictions[,eventid] = tsprediction
	}
	return(predictions)
}



event.ids <- paste("E",1:12,sep = "")
idlist = as.character(read.table("data/hollywood.idlist")[,1])
labels = read.matrix("data/hollywood.labels")
splits = as.matrix(read.matrix("data/hollywood.splits"))

nfeat = read.matrix("data/hollywood.fet.pca")
#this is how nfeat is generated
#feat = read.matrix("data/dtraj.feat")
#dim = nrow(feat)
#pca = prcomp(feat, center = TRUE)
#nfeat = pca$x

#tuning the parameters
lu = "hollywood2+adaboost"
iter = c(10,20,50,80)
loss = c("exponential","logistic")

para.table = expand.grid(iter = iter, loss = loss)
para.map = matrix(0, nrow = nrow(para.table), ncol = length(event.ids)) 

for(i in 1:nrow(para.table)) {
	predictions = train_adaboost(1, nfeat, iter=para.table[i,1], loss = para.table[i,2])
	testlabels = labels[which(splits[,1]==1),]
	para.map[i,] = mean(evaluate.map(predictions, testlabels))
}
print(cbind(para.table, para.map))

#you should be able to see the results like
#  iter        loss  para.map
#1   10 exponential 0.3354384
#2   20 exponential 0.3608609
#3   50 exponential 0.4010366
#4   80 exponential 0.4113735
#5   10    logistic 0.3384905
#6   20    logistic 0.3700407
#7   50    logistic 0.3854823
#8   80    logistic 0.3879747

predictions = train_adaboost(1, nfeat, iter=80, loss = "exponential")
testlabels = labels[which(splits[,1]==1),]
print("MAP:")
print(evaluate.map(predictions, testlabels))
