library(Matrix)
library(tseries)
library(randomForest)
source("http://lu-library.googlecode.com/svn/trunk/evaluation.r")


#random forest
train_randforest <- function(splitid, nfeat, classwt = c(0.08,1), ntree=16, mtry = 16) {
	testidx = which(splits[,splitid]==1)
	trainidx = which(splits[,splitid]==0)
	print(splitid)
	
	trmat <- as.matrix(nfeat[trainidx,])
	tsmat <- as.matrix(nfeat[testidx,])
	
	set.seed(0)
	predictions <-  matrix(0, nrow = length(testidx), ncol = length(event.ids))
	for(eventid in 1:length(event.ids)) {
		trlabels = labels[trainidx, eventid]
		set.seed(0) 
		model <- randomForest(x = trmat, y = factor(trlabels), classwt = c(0.2,1), ntree=ntree, mtry=mtry)
		pred<- predict(model, tsmat, type="vote")
		classes = attributes(pred)$dimnames[[2]]
		posclassidx = which(classes=="1")
		tsprediction = as.numeric(pred[,posclassidx])
		predictions[,eventid] = tsprediction
	}
	return(predictions)
}



event.ids <- paste("E",1:12,sep = "")
idlist = as.character(read.table("data/hollywood.idlist")[,1])
labels = read.matrix("data/hollywood.labels")
splits = as.matrix(read.matrix("data/hollywood.splits"))

nfeat = read.matrix("data/hollywood.fet.pca")
#this is how nfeat is generated
#feat = read.matrix("data/dtraj.feat")
#dim=nrow(feat)
#pca = prcomp(feat, center = TRUE)
#nfeat = pca$x


#tuning the parameters
lu = "hollywood2+randomforest"
ntree = c(128,512,1024)
mtry = c(32,64,128)
para.table = expand.grid(ntree = ntree, mtry = mtry)
para.map = matrix(0, nrow = nrow(para.table), ncol = length(event.ids)) 

for(i in 1:nrow(para.table)) {
	predictions = train_randforest(1, nfeat, ntree=para.table[i,1], mtry = para.table[i,2])
	testlabels = labels[which(splits[,1]==1),]
	para.map[i,] = mean(evaluate.map(predictions, testlabels))
}
print(cbind(para.table, para.map))

#you should be able to see the results like
#  ntree mtry  para.map
#1   128   32 0.2329447
#2   512   32 0.2429325
#3  1024   32 0.2439838
#4   128   64 0.2382421
#5   512   64 0.2575630
#6  1024   64 0.2615122
#7   128  128 0.2724160
#8   512  128 0.2787697
#9  1024  128 0.2819644

predictions = train_randforest(1, nfeat, ntree=1024, mtry = 128)
testlabels = labels[which(splits[,1]==1),]
print("MAP:")
print(evaluate.map(predictions, testlabels))


