library(Matrix)
library(tseries)
library(kernlab)
source("http://lu-library.googlecode.com/svn/trunk/evaluation.r")
source("../selfpace_lib.r")

selfpaced.single <- function(splitid, initialvals = NULL, pos.stepsize = 1, neg.stepsize = 100, C = 0.5) {
	print(splitid)
	trainidx = which(splits[,splitid]==0)
	testidx = which(splits[,splitid]==1)
	
	#the list storing the result for each iteration
	valpredmatrix = list()
	evlpredmatrix = list()	
	for(iter in 1:30) {
		valpredmatrix[[iter]] = matrix(0, nrow = length(trainidx), ncol = length(event.ids))
		evlpredmatrix[[iter]] = matrix(0, nrow = length(testidx), ncol = length(event.ids))
	}
	
	#the statring values
	initial.posidx = list()
	initial.negidx = list()
	
	#the selected positive and negative samples for all iterations
	iters.posidx = list()	
	iters.negidx = list()
	
	
	for(eventid in 1:length(event.ids)) {#for each event
		#validation set is the same as the training set
		dev_labels = labels[trainidx, eventid]
		val_labels = dev_labels					
		
		this.training.posidx = intersect(which(splits[,splitid]==0), which(labels[, eventid]==1))
		this.training.negidx = intersect(which(splits[,splitid]==0), which(labels[, eventid]==0))
		
		pos.stepsize = pos.stepsize
		neg.stepsize = neg.stepsize
		
		#self-pace parameters
		pos.lambda1_rank = pos.stepsize
		neg.lambda1_rank = neg.stepsize
		
		if(is.null(initialvals)) {
			#when the starting value is missing, use randomly sampled positives and negatives.
			this.training.posidx = sample(this.training.posidx, pos.lambda1_rank)
			this.training.negidx = sample(this.training.negidx, neg.lambda1_rank)
		} else {
			#when the starting value is set, use it.
			this.training.posidx = initialvals$pos[[eventid]]
			this.training.negidx = initialvals$neg[[eventid]]
		}
		
		#store starting values
		initial.posidx[[eventid]] = this.training.posidx
		initial.negidx[[eventid]] = this.training.negidx
		
		iters.posidx[[eventid]] = list()
		iters.negidx[[eventid]] = list()
		
		for(iter in 1:30) {
			print(paste("action", eventid, "iter ", iter))
			print(paste("-----selected neg=", length(this.training.negidx), " pos=", length(this.training.posidx), sep = ""))
			
			#store the posidx and negidx used in this iteration
			iters.posidx[[eventid]][[iter]] = this.training.posidx
			iters.negidx[[eventid]][[iter]] = this.training.negidx
			
			thistraining.idx = c(this.training.posidx, this.training.negidx)
			thistraining.labels = c(replicate(length(this.training.posidx),1), replicate(length(this.training.negidx),0))
			
			
			if(length(which((labels[thistraining.idx,eventid] == thistraining.labels)==FALSE))!=0) {
				#check whether the labels are consistent (for debug purposes)
				print(paste("Warning: incosistent labels found, please check: action", eventid, "iter ", iter))
				return
			}
			
			#off-the-shelf SVM training
			thistrmat = as.kernelMatrix(kernels[thistraining.idx,thistraining.idx])
			model = ksvm(x=thistrmat, y=thistraining.labels, type = "C-svc", C = C)
			
			#SVM validates on the validation set
			thisdevmat = as.kernelMatrix(kernels[trainidx,thistraining.idx[SVindex(model)]])
			valprediction = predict(model, thisdevmat, type = "decision")
			trprediction = valprediction	#in this problem the whole training set is used as the validation set.

			#SVM validates on the test set
			thistsmat = as.kernelMatrix(kernels[testidx,thistraining.idx[SVindex(model)]])
			tsprediction = predict(model, thistsmat, type = "decision")
			
			#self-paced learning
			selectedidx = selfpace1.rank(trprediction, dev_labels, pos.lambda1_rank, neg.lambda1_rank)

			#change the parameter accoding to the step size
			pos.lambda1_rank = pos.lambda1_rank + pos.stepsize
			neg.lambda1_rank = neg.lambda1_rank + neg.stepsize
			
			#store the prediction for this iteration
			valpredmatrix[[iter]][,eventid] = as.numeric(valprediction)
			evlpredmatrix[[iter]][,eventid] = tsprediction
			
			#set the selected positive and negative for the next iteration.
			this.training.negidx = trainidx[unique(selectedidx$neg)]
			this.training.posidx = trainidx[unique(selectedidx$pos)]
		}
	}
	
	initialvals = list("pos" = initial.posidx, "neg" = initial.negidx)	#starting values
	results <- list("valpredmatrix" = valpredmatrix, "evlpredmatrix" = evlpredmatrix, "initialvals" = initialvals, "iterposidx" = iters.posidx, "iternegidx" = iters.negidx)
	return(results)
}

evaluate.run <- function(splitid, package, mode = "aps") {
	#calculate map for the return results in "package"
	alltrain = which(splits[,splitid]==0)
	aps = matrix(0, nrow=1, ncol = 12)
	trainingidx = alltrain
	validx = trainingidx
	testidx = which(splits[,splitid]==1)
	testlabels = labels[testidx,]
	traininglabels = labels[trainingidx,]

	devaps = matrix(0, nrow=30, ncol=12)	#maps on the validation set
	testaps = matrix(0, nrow=30, ncol=12)	#maps on the test set

	#calculate ap for each event according to the ap on the validation set
	for(iterid in 1:30) {
		testaps[iterid,] = evaluate.map(package$evlpredmatrix[[iterid]], testlabels)
		devaps[iterid,] = evaluate.map(package$valpredmatrix[[iterid]], traininglabels)
	}

	bestiters = findbestiter2(devaps)
	print("##########best iter ###############")
	print(bestiters)
	
	#calculate the map
	for(eventid in 1:12) {
		aps[splitid, eventid] = testaps[bestiters[eventid],eventid]
	}
	
	if(mode == "aps") {
		return(t(aps))
	} else if( mode == "bestiters") {
		return(bestiters)
	} else if( mode == "devevl") {
		return(list("devaps" = devaps, "evlaps" = testaps))
	}
}


event.ids <- paste("E",1:12,sep = "")
kernels = read.matrix("data/dev.matrix")
kernels = kernels * 2	#our tool divides pre-calculated kernel by 2 and here we get it back

idlist = as.character(read.table("data/hollywood.idlist")[,1])
labels = read.matrix("data/hollywood.labels")
splits = as.matrix(read.matrix("data/hollywood.splits"))


#tuning the parameters
lu = "hollywood2+spl"



#######################################################################
#######################################################################
### if you cannot get the map in our paper                          ###
### probably due to the randomness in starting values/clustering    ###
### please run the code below to load our starting values           ###
### which can generate the same output                              ###
#######################################################################
#######################################################################
#1) load the starting values
load("hollywood2.spl.bestrun.obj")
myrun = hollywood2.spl.bestrun

#2) rerun the results
bestrun = selfpaced.single(1, initialvals = myrun$initialvals, pos.stepsize = 3, neg.stepsize =150)

#3) evaluate the result
aps = evaluate.run(1, bestrun)
print(cbind(1:length(aps), aps))
print(mean(aps))

